#include "alphaqed.h"

const double alphaqed::a0QED = 1./128.89;
const double alphaqed::equarkup=4./9.;
const double alphaqed::equarkdn=1./9.;
const double alphaqed::nups=3.;
const double alphaqed::ndns=3.;
const double alphaqed::nleps=3.;
const double alphaqed::NF2=3.*(nups*equarkup+ndns*equarkdn)+nleps;
const double alphaqed::NF4=3.*(nups*pow(equarkup,2)+ndns*pow(equarkdn,2))+nleps;
// for 3-loop
const double alphaqed::NF6=3.*(nups*pow(equarkup,3)+ndns*pow(equarkdn,3))+nleps;  //!3.*(3.*(2./3.)**6+3.*(1./3.)**6)+3.
const double alphaqed::NF3=3.*(nups*pow(sqrt(equarkup),3)+ndns*pow(sqrt(equarkdn),3))+nleps; //3.*(3.*(2./3.)**3+3.*(1./3.)**3)+3.;

const double alphaqed::beta0QED=-4.*NF2/12.;
const double alphaqed::beta1QED=-4.*NF4/16.;

double alphaqed::calc(double q2, int nloop)
{
  //Solve Eq.(26) of 1311.1654 taking alphaQED(MZ=91.1876)=1./128.89
  //The output is alpha_QED/(4*Pi)

  double b0 = -4.*NF2/12./M_PI;
  double b1 = -6.*NF4/24./(pow(M_PI,2));
  double b2 = (2.*NF6+22.*NF3*NF3/9.)/128./(pow(M_PI,3));
      
  double t = 2.*log(sqrt(q2)/91.1876);

  double alfa;
  if (nloop == 1) 
    alfa = a0QED*(1.-a0QED*b0*t);
  else if (nloop == 2) 
    alfa = a0QED*(1.-a0QED*b0*t+pow(a0QED,2)*(pow(b0,2)*pow(t,2)-b1*t));
  else if (nloop == 3)
    alfa = a0QED*(1.-a0QED*b0*t+pow(a0QED,2)*(pow(b0,2)*pow(t,2)-b1*t)
		  + pow(a0QED,3)*(-pow(b0,3)*pow(t,3)+5.*b1*b0*pow(t,2)/2.-b2*t));

  return alfa/4./M_PI;
}
