#include "gamma2.h"
#include "constants.h"
#include "psi.h"
#include "melfun.h"

using namespace constants;

complex <double> gamma2pqqps(complex <double> N, double NF)
{
  complex <double> N1 = N + 1.;
  complex <double> N2 = N + 2.;
  complex <double> NM = N - 1.;

  complex <double> NS = N * N;
  complex <double> NT = NS * N;
  complex <double> NFO = NT * N;
  complex <double> NFI = NFO * N;

  complex <double> NMS = NM * NM;
  complex <double> N1S = N1 * N1;
  complex <double> N1T = N1S * N1;
  complex <double> N2S = N2 * N2;
  complex <double> N2T = N2S * N2;
  
  return TF*NF * CF * ((5.* NFI + 32.* NFO + 49.* NT + 38.* NS + 28.* N + 8.) / (NM * NT * N1T * N2S) * 2.) * 4.;
}

complex <double> gamma2pqg(complex <double> N, double NF)
{
  complex <double> S1 = cpsi0(N+1.) + euler;
  complex <double> S2 = zeta2 - cpsi(1,N+1.);
  complex <double> SSTR2P = zeta2 - cpsi(1,(N+2.)/2.);

  complex <double> N1 = N + 1.;
  complex <double> N2 = N + 2.;
  complex <double> NM = N - 1.;

  complex <double> NS = N * N;
  complex <double> NT = NS * N;
  complex <double> NFO = NT * N;
  complex <double> NFI = NFO * N;
  complex <double> NSI = NFI * N;
  complex <double> NSE = NSI * N;
  complex <double> NE = NSE * N;
  complex <double> NN = NE * N;

  complex <double> NMS = NM * NM;
  complex <double> N1S = N1 * N1;
  complex <double> N1T = N1S * N1;
  complex <double> N2S = N2 * N2;
  complex <double> N2T = N2S * N2;

  complex <double> PQGA = (-2.* S1 * S1 + 2.* S2 - 2.* SSTR2P) 
    * (NS + N + 2.) / (N * N1 * N2) 
    + (8.* S1 * (2.* N + 3.)) / (N1S * N2S)
    + 2.* (NN + 6.* NE + 15. * NSE + 25.* NSI + 36.* NFI
	   + 85.* NFO + 128.* NT + 104.* NS + 64.* N + 16.)
    / (NM * NT * N1T * N2T);
  complex <double> PQGB = (2.* S1 * S1 - 2.* S2 + 5.) * (NS + N + 2.)
    / (N * N1 * N2)   -   4.* S1 / NS
    + (11.* NFO + 26.* NT + 15.* NS + 8.* N + 4.)
    / (NT * N1T * N2);
  
  return (TF*4 * (CA * PQGA + CF * PQGB) * 4.);
}


complex <double> gamma2pgq(complex <double> N, double NF)
{
  complex <double> S1 = cpsi0(N+1.) + euler;
  complex <double> S2 = zeta2 - cpsi(1,N+1.);
  complex <double> SSTR2P = zeta2 - cpsi(1,(N+2.)/2.);

  complex <double> N1 = N + 1.;
  complex <double> N2 = N + 2.;
  complex <double> NM = N - 1.;

  complex <double> NS = N * N;
  complex <double> NT = NS * N;
  complex <double> NFO = NT * N;
  complex <double> NFI = NFO * N;
  complex <double> NSI = NFI * N;
  complex <double> NSE = NSI * N;
  complex <double> NE = NSE * N;
  complex <double> NN = NE * N;

  complex <double> NMS = NM * NM;
  complex <double> N1S = N1 * N1;
  complex <double> N1T = N1S * N1;
  complex <double> N2S = N2 * N2;
  complex <double> N2T = N2S * N2;

  complex <double> PGQA = (- S1 * S1 + 5.* S1 - S2) * (NS + N + 2.) 
    / (NM * N * N1)  -  2.* S1 / N1S
    - (12.* NSI + 30.* NFI + 43.* NFO + 28.* NT - NS
       - 12.* N - 4.) / (2.* NM * NT * N1T);
  complex <double> PGQB = (S1*S1 + S2 - SSTR2P) * (NS + N + 2.) / (NM * N * N1)
    - S1 * (17.* NFO + 41.* NS - 22.* N - 12.) 
    / (3.* NMS * NS * N1)
    + (109.* NN + 621.* NE + 1400.* NSE + 1678.* NSI
       + 695.* NFI - 1031.* NFO - 1304.* NT - 152.* NS
       + 432.* N + 144.) / (9.* NMS * NT * N1T * N2S);
  complex <double> PGQC = (S1 - 8./3.) * (NS + N + 2.) / (NM * N * N1)  +  1./ N1S;
  PGQC = 4./3.* PGQC;

  return ( CF*CF * PGQA + CF*CA * PGQB + TF*NF * CF * PGQC) * 4.;
}

complex <double> gamma2pgg(complex <double> N, double NF)
{
  complex <double> S1 = cpsi0(N+1.) + euler;
  complex <double> S2 = zeta2 - cpsi(1,N+1.);

  complex <double> N1 = N + 1.;
  complex <double> N2 = N + 2.;
  complex <double> NM = N - 1.;

  complex <double> NS = N * N;
  complex <double> NT = NS * N;
  complex <double> NFO = NT * N;
  complex <double> NFI = NFO * N;
  complex <double> NSI = NFI * N;
  complex <double> NSE = NSI * N;
  complex <double> NE = NSE * N;
  complex <double> NN = NE * N;

  complex <double> NMS = NM * NM;
  complex <double> N1S = N1 * N1;
  complex <double> N1T = N1S * N1;
  complex <double> N2S = N2 * N2;
  complex <double> N2T = N2S * N2;

  complex <double> SPMOM = fun6(N-1.); //Mellin transform of: Li2(x)/(1+x)  
  complex <double> SLC = - 5./8. * zeta3;
  complex <double> SLV = - zeta2/2.* (cpsi0(N1/2.) - cpsi0(N/2.)) + S1/NS + SPMOM;
  complex <double> SSCHLM = SLC - SLV;
  complex <double> SSTR2M = zeta2 - cpsi(1,N1/2.);
  complex <double> SSTR3M = 0.5 * cpsi(2,N1/2.) + zeta3;
  complex <double> SSCHLP = SLC + SLV;
  complex <double> SSTR2P = zeta2 - cpsi(1,N2/2.);
  complex <double> SSTR3P = 0.5 * cpsi(2,N2/2.) + zeta3;
  
  complex <double> PGGA = - (2.* NFI + 5.* NFO + 8.* NT + 7.* NS - 2.* N - 2.)
    * 8.* S1 / (NMS * NS * N1S * N2S) -  67./9.* S1 + 8./3.
    - 4.* SSTR2P * (NS + N + 1.) / (NM * N * N1 * N2)
    + 2.* S1 * SSTR2P - 4.* SSCHLP + 0.5 * SSTR3P
    + (457.* NN + 2742.* NE + 6040.* NSE + 6098.* NSI
       + 1567.* NFI - 2344.* NFO - 1632.* NT + 560.* NS
       + 1488.* N + 576.) / (18.* NMS * NT * N1T * N2T);
  complex <double> PGGB = (38.* NFO + 76.* NT + 94.* NS + 56.* N + 12.) * (-2.)
    / (9.* NM * NS * N1S * N2)  +  20./9.* S1  -  4./3.;
  complex <double> PGGC = (2.* NSI + 4.* NFI + NFO - 10.* NT - 5.* NS - 4.* N
			   - 4.) * (-2.) / (NM * NT * N1T * N2)  -  1.;

  return ( CA*CA * PGGA + TF*NF * (CA * PGGB + CF * PGGC) ) * 4.;
}

complex <double> gamma2pnsp(complex <double> N, double NF)
{
  complex <double> S1 = cpsi0(N+1.) + euler;
  complex <double> S2 = zeta2 - cpsi(1,N+1.);

  complex <double> N1 = N + 1.;
  complex <double> N2 = N + 2.;
  complex <double> NM = N - 1.;

  complex <double> NS = N * N;
  complex <double> NT = NS * N;
  complex <double> NFO = NT * N;
  complex <double> NFI = NFO * N;
  complex <double> NSI = NFI * N;
  complex <double> NSE = NSI * N;
  complex <double> NE = NSE * N;
  complex <double> NN = NE * N;

  complex <double> NMS = NM * NM;
  complex <double> N1S = N1 * N1;
  complex <double> N1T = N1S * N1;
  complex <double> N2S = N2 * N2;
  complex <double> N2T = N2S * N2;

  complex <double> SPMOM = fun6(N-1.); //Mellin transform of: Li2(x)/(1+x)  
  complex <double> SLC = - 5./8. * zeta3;
  complex <double> SLV = - zeta2/2.* (cpsi0(N1/2.) - cpsi0(N/2.)) + S1/NS + SPMOM;
  complex <double> SSCHLM = SLC - SLV;
  complex <double> SSTR2M = zeta2 - cpsi(1,N1/2.);
  complex <double> SSTR3M = 0.5 * cpsi(2,N1/2.) + zeta3;
  complex <double> SSCHLP = SLC + SLV;
  complex <double> SSTR2P = zeta2 - cpsi(1,N2/2.);
  complex <double> SSTR3P = 0.5 * cpsi(2,N2/2.) + zeta3;
  
  complex <double> PNMA = ( 16.* S1 * (2.* N + 1.) / (NS * N1S) +
			    16.* (2.* S1 - 1./(N * N1)) * ( S2 - SSTR2M ) +
			    64.* SSCHLM + 24.* S2 - 3. - 8.* SSTR3M -
			    8.* (3.* NT + NS -1.) / (NT * N1T) +
			    16.* (2.* NS + 2.* N +1.) / (NT * N1T) ) * (-0.5);
  complex <double> PNPA = ( 16.* S1 * (2.* N + 1.) / (NS * N1S) +
			    16.* (2.* S1 - 1./(N * N1)) * ( S2 - SSTR2P ) +
			    64.* SSCHLP + 24.* S2 - 3. - 8.* SSTR3P -
			    8.* (3.* NT + NS -1.) / (NT * N1T) -
			    16.* (2.* NS + 2.* N +1.) / (NT * N1T) ) * (-0.5);
  complex <double> PNSB = ( S1 * (536./9. + 8.* (2.* N + 1.) / (NS * N1S)) -
			    (16.* S1 + 52./3.- 8./(N * N1)) * S2 - 43./6. -
			    (151.* NFO + 263.* NT + 97.* NS + 3.* N + 9.) *
			    4./ (9.* NT * N1T) ) * (-0.5);
  complex <double> PNSC = ( -160./9.* S1 + 32./3.* S2 + 4./3. +
			    16.* (11.* NS + 5.* N - 3.) / (9.* NS * N1S) )*(-0.5);
  
  return CF * ((CF-CA/2.)* PNPA + CA* PNSB + TF*NF* PNSC);
}
complex <double> gamma2pnsm(complex <double> N, double NF)
{
  complex <double> S1 = cpsi0(N+1.) + euler;
  complex <double> S2 = zeta2 - cpsi(1,N+1.);

  complex <double> N1 = N + 1.;
  complex <double> N2 = N + 2.;
  complex <double> NM = N - 1.;

  complex <double> NS = N * N;
  complex <double> NT = NS * N;
  complex <double> NFO = NT * N;
  complex <double> NFI = NFO * N;
  complex <double> NSI = NFI * N;
  complex <double> NSE = NSI * N;
  complex <double> NE = NSE * N;
  complex <double> NN = NE * N;

  complex <double> NMS = NM * NM;
  complex <double> N1S = N1 * N1;
  complex <double> N1T = N1S * N1;
  complex <double> N2S = N2 * N2;
  complex <double> N2T = N2S * N2;

  complex <double> SPMOM = fun6(N-1.); //Mellin transform of: Li2(x)/(1+x)  
  complex <double> SLC = - 5./8. * zeta3;
  complex <double> SLV = - zeta2/2.* (cpsi0(N1/2.) - cpsi0(N/2.)) + S1/NS + SPMOM;
  complex <double> SSCHLM = SLC - SLV;
  complex <double> SSTR2M = zeta2 - cpsi(1,N1/2.);
  complex <double> SSTR3M = 0.5 * cpsi(2,N1/2.) + zeta3;
  complex <double> SSCHLP = SLC + SLV;
  complex <double> SSTR2P = zeta2 - cpsi(1,N2/2.);
  complex <double> SSTR3P = 0.5 * cpsi(2,N2/2.) + zeta3;
  
  complex <double> PNMA = ( 16.* S1 * (2.* N + 1.) / (NS * N1S) +
			    16.* (2.* S1 - 1./(N * N1)) * ( S2 - SSTR2M ) +
			    64.* SSCHLM + 24.* S2 - 3. - 8.* SSTR3M -
			    8.* (3.* NT + NS -1.) / (NT * N1T) +
			    16.* (2.* NS + 2.* N +1.) / (NT * N1T) ) * (-0.5);
  complex <double> PNPA = ( 16.* S1 * (2.* N + 1.) / (NS * N1S) +
			    16.* (2.* S1 - 1./(N * N1)) * ( S2 - SSTR2P ) +
			    64.* SSCHLP + 24.* S2 - 3. - 8.* SSTR3P -
			    8.* (3.* NT + NS -1.) / (NT * N1T) -
			    16.* (2.* NS + 2.* N +1.) / (NT * N1T) ) * (-0.5);
  complex <double> PNSB = ( S1 * (536./9. + 8.* (2.* N + 1.) / (NS * N1S)) -
			    (16.* S1 + 52./3.- 8./(N * N1)) * S2 - 43./6. -
			    (151.* NFO + 263.* NT + 97.* NS + 3.* N + 9.) *
			    4./ (9.* NT * N1T) ) * (-0.5);
  complex <double> PNSC = ( -160./9.* S1 + 32./3.* S2 + 4./3. +
			    16.* (11.* NS + 5.* N - 3.) / (9.* NS * N1S) )*(-0.5);
  
  return CF * ((CF-CA/2.)* PNMA + CA* PNSB + TF*NF* PNSC);
}
complex <double> gamma2pnsv(complex <double> N, double NF)
{
  complex <double> S1 = cpsi0(N+1.) + euler;
  complex <double> S2 = zeta2 - cpsi(1,N+1.);

  complex <double> N1 = N + 1.;
  complex <double> N2 = N + 2.;
  complex <double> NM = N - 1.;

  complex <double> NS = N * N;
  complex <double> NT = NS * N;
  complex <double> NFO = NT * N;
  complex <double> NFI = NFO * N;
  complex <double> NSI = NFI * N;
  complex <double> NSE = NSI * N;
  complex <double> NE = NSE * N;
  complex <double> NN = NE * N;

  complex <double> NMS = NM * NM;
  complex <double> N1S = N1 * N1;
  complex <double> N1T = N1S * N1;
  complex <double> N2S = N2 * N2;
  complex <double> N2T = N2S * N2;

  complex <double> SPMOM = fun6(N-1.); //Mellin transform of: Li2(x)/(1+x)  
  complex <double> SLC = - 5./8. * zeta3;
  complex <double> SLV = - zeta2/2.* (cpsi0(N1/2.) - cpsi0(N/2.)) + S1/NS + SPMOM;
  complex <double> SSCHLM = SLC - SLV;
  complex <double> SSTR2M = zeta2 - cpsi(1,N1/2.);
  complex <double> SSTR3M = 0.5 * cpsi(2,N1/2.) + zeta3;
  complex <double> SSCHLP = SLC + SLV;
  complex <double> SSTR2P = zeta2 - cpsi(1,N2/2.);
  complex <double> SSTR3P = 0.5 * cpsi(2,N2/2.) + zeta3;
  
  complex <double> PNMA = ( 16.* S1 * (2.* N + 1.) / (NS * N1S) +
			    16.* (2.* S1 - 1./(N * N1)) * ( S2 - SSTR2M ) +
			    64.* SSCHLM + 24.* S2 - 3. - 8.* SSTR3M -
			    8.* (3.* NT + NS -1.) / (NT * N1T) +
			    16.* (2.* NS + 2.* N +1.) / (NT * N1T) ) * (-0.5);
  complex <double> PNPA = ( 16.* S1 * (2.* N + 1.) / (NS * N1S) +
			    16.* (2.* S1 - 1./(N * N1)) * ( S2 - SSTR2P ) +
			    64.* SSCHLP + 24.* S2 - 3. - 8.* SSTR3P -
			    8.* (3.* NT + NS -1.) / (NT * N1T) -
			    16.* (2.* NS + 2.* N +1.) / (NT * N1T) ) * (-0.5);
  complex <double> PNSB = ( S1 * (536./9. + 8.* (2.* N + 1.) / (NS * N1S)) -
			    (16.* S1 + 52./3.- 8./(N * N1)) * S2 - 43./6. -
			    (151.* NFO + 263.* NT + 97.* NS + 3.* N + 9.) *
			    4./ (9.* NT * N1T) ) * (-0.5);
  complex <double> PNSC = ( -160./9.* S1 + 32./3.* S2 + 4./3. +
			    16.* (11.* NS + 5.* N - 3.) / (9.* NS * N1S) )*(-0.5);
  
  return CF * ((CF-CA/2.)* PNMA + CA* PNSB + TF*NF* PNSC);
}

complex <double> gamma2pqq(complex <double> N, double NF)
{
  return gamma2pqqps(N,NF)+gamma2pnsp(N,NF);
}
