#include "dilog.h"
#include "constants.h"

complex <double> dilog( complex <double> x)
{
  double zero=1.E-8;
  double xr = std::real(x);
  double xi = std::imag(x);
  double r2 = xr*xr+xi*xi;
  std::complex<double> DILOG(0.,0.);
  complex <double> y;
  if (r2<=zero) DILOG = x + x*x/4.;
  double rr=xr/r2;

  if ( r2 == 1. && xi == 0. )
    {
      if (xr==1.)  DILOG = (constants::zeta2);
      else DILOG = -constants::zeta2/2.;//+0.i;
    }
  else if ( r2 > 1. && rr > 0.5)
	{	  
	  y = (x-1.)/x;
	  DILOG = cli2(y) + constants::zeta2 - log(x)*log(1.-x)+0.5*log(x)*log(x);
	}
  else if ( r2 > 1. && rr < 0.5)
    {
      y = 1./x;
      DILOG = -cli2(y) - constants::zeta2 - 0.5*log(-x)*log(-x);
    }
  else if ( r2 < 1. && xr > 0.5) 
    {
      y = 1.-x;
      DILOG = -cli2(y) + constants::zeta2 - log(x)*log(1.-x);
    }
  else if ( r2 < 1. && xr < 0.5 ) 
    {
      y = x;
      DILOG = cli2(y);
    }    							 
  // Implementazione in GSL pronta ad essere usata
  /*
  double r = std::abs(z);
  double theta = std::arg(z);
  gsl_sf_result re, im;
  gsl_sf_complex_dilog_e(r, theta, &re, &im);
  //  std::cout << re.val << " + " << im.val << "i" << std::endl;
  std::complex<double> resultcomplex (re.val,im.val);
  return resultcomplex;
  */
  // std::complex<double> resultcomplex (0,0);
  return DILOG;
}

complex <double> cli2(complex <double> x)
{//  c--taylor-expansion for complex dilogarithm (spence-function)
  //b2(           1 )  =  -0.25000000000000000     
  //b2(           2 )  =    2.7777777777777776E-002
  //b2(           3 )  =    0.0000000000000000     
  //b2(           4 )  =   -2.7777777777777778E-004
  //b2(           5 )  =    0.0000000000000000     
  //b2(           6 )  =    4.7241118669690098E-006
  //b2(           7 )  =    0.0000000000000000     
  //b2(           8 )  =   -9.1857730746619641E-008
  //b2(           9 )  =    0.0000000000000000     
  //b2(          10 )  =    1.8978869988971001E-009
  //b2(          11 )  =    0.0000000000000000     
  //b2(          12 )  =   -4.0647616451442256E-011
  //b2(          13 )  =    0.0000000000000000     
  //b2(          14 )  =    8.9216910204564533E-013
  //b2(          15 )  =    0.0000000000000000     
  //b2(          16 )  =   -1.9939295860721074E-014
  //b2(          17 )  =    0.0000000000000000     
  //b2(          18 )  =    4.5189800296199183E-016
  //double b [] = {0.0,-1./2., 1./6., 0., -1./30., 0., 1./42., 0., -1./30., 0., 5./66., 0., -691./2730., 0., 7./6., 0., -3617./510., 0., 43867./798.};
  double b2 [] = {0.0, -0.25000000000000000, 2.7777777777777776E-002, 0., -2.7777777777777778E-004, 0., 4.7241118669690098E-006, 0., -9.1857730746619641E-008, 0., 1.8978869988971001E-009, 0.,-4.0647616451442256E-011, 0., 8.9216910204564533E-013, 0., -1.9939295860721074E-014, 0., 4.5189800296199183E-016};
  int nber = 18;
  int n = nber-1;
  complex <double> tmp = b2[nber];
  complex <double> z = -std::log(1.-x);
  //cli2=b2(nber)
  for(int i=n;i==1;i--) tmp = z*tmp+b2[i];
  tmp = z*z*tmp+z;
  //  do 111 i=n,1,-1
  //       cli2=z*cli2+b2(i)
  //111   continue
  //      cli2=z**2*cli2+z
  //      return 
  //      end
  return tmp;
}
